package celular;

/**
 * Universidade Federal de Pernambuco
 * Centro de Informtica
 * Disciplina: Introducao a Programao
 * Projeto Celular
 *
 * Tipo: PrevisaoTexto
 *
 * @author: Alberto Csar C. Frana     (accf2@cin.ufpe.br)
 * @author: Luciana Pereira Oliveira    (lpo@cin.ufpe.br)
 * @author: Milena Rocha Leite          (mrl@cin.ufpe.br)
 *
 * @since 2002
 * @version 1.2
 */
import celular.palavras.CadastroPalavras;
import celular.palavras.IListaPalavras;
import celular.palavras.Palavra;
import celular.palavras.excecoes.PalavraJaExisteException;
import celular.palavras.excecoes.PalavraNaoExisteException;
import celular.util.excecoes.ParametroNuloException;



public class PrevisaoTexto implements IPrevisaoTexto {

	private CadastroPalavras vocabulario;

	public PrevisaoTexto(CadastroPalavras c){
		this.vocabulario = c;
	}
   /**
	* Mtodo que exclui uma palavra do dicionario
	*
	* @param Palavra  palvra a ser excluda
	*/
	public void excluir(Palavra palavra) throws PalavraNaoExisteException{

		vocabulario.excluir(palavra);
	}
  /**
   * Mtodo que indica se uma palavra existe no dicionario
   *
   * @param  Palavra   palavra a ser procurada
   * @return boolean   boleano que informa se a palavra existe ou nao na colecao
   */
	public boolean existe(Palavra palavra) throws PalavraNaoExisteException{
		  return vocabulario.existe(palavra);
   }   
   /**
	* Mtodo que retorna todas as palavras do dicionario que possuem uma das
	* letras passadas da posicao indicada
	*
	* @param  char[]         characteres a serem verificados
	* @return ListaPalavras  lista de palavras que possuem um dos caracteres
	*                          passados
	*/
	public IListaPalavras filtrar(char[] letras,int posicaoAtual)throws PalavraNaoExisteException{
		return vocabulario.filtrar(letras,posicaoAtual);
   }   
	/**
	 * Mtodo que insere uma palavra no dicionario
	 *
	 * @param Palvra   plavra a ser inserida
	 */
	public void inserir(Palavra palavra) throws PalavraJaExisteException,
												PalavraNaoExisteException{
		vocabulario.inserir(palavra);
	}
}
