package celular.interfaceTextual;

/**
  * Universidade Federal de Pernambuco
  * Centro de Informtica
  * Disciplina: Introducao a Programao
  * Projeto Celular
  *
  * Tipo: InterfacePrevisaoTexto
  *
  * @author: Alberto Csar C. Frana     (accf2@cin.ufpe.br)
  * @author: Luciana Pereira Oliveira    (lpo@cin.ufpe.br)
  * @author: Milena Rocha Leite          (mrl@cin.ufpe.br)
  *
  * @since 2002
  * @version 1.1
  */
import celular.palavras.IListaPalavras;
import celular.palavras.dados.PalavrasList;
import celular.palavras.excecoes.PalavraJaExisteException;
import celular.palavras.excecoes.PalavraNaoExisteException;
import celular.util.excecoes.ParametroNuloException;
import celular.IPrevisaoTexto;
import celular.PrevisaoTexto;
import celular.palavras.CadastroPalavras;
import celular.palavras.Palavra;
import celular.palavras.PalavraSimples;
import celular.palavras.PalavraComposta;
import celular.palavras.dados.PalavrasArray;
import celular.util.Console;



public class InterfacePrevisaoTexto {


   private IPrevisaoTexto celular;
  //usado para saber a posicao do digito
   private int indiceDigito;
   //palvrs filtradas
   private IListaPalavras filtro;

   InterfacePrevisaoTexto() {

		this.filtro = new PalavrasList();
		indiceDigito = 1;
		//para usar vector descomente a linha abaixo e
		//comente a instaciacao usando array
//		IListaPalavras lista = new PalavrasList();

		//Usando concorrencia
		IListaPalavras lista = new PalavrasArray();
		
		String palavrasSimples[] = { "abri", "cesar", "ajuda", "aliado", "casa", "bar", "barco",
		                    "belo", "bola",  "boneca", "branco","brasil","cabelo",
		                    "carro", "castelo" ,"dama","data", "duvida", "elefante",
		                    "errado","errar","escola","familia","farmacia", "fonte",
		                     "formiga","foto","funeral","futebol","gabarito","gaiola",
		                    "garfo","gordo","grafico","grupo","guarda","guerra","habito",
		                    "haver","hematoma","heroi","hipotese","historia","homem",
		                    "hora","humano","ideal","igual","imaginacao","imobiliaria",
		                     "impessoal","indice","ingles","invalido","jacare","jogo",
		                      "justica","lado","lago","lanterna","lei","limao", "limite",
		                      "limpo","livre","livro","lobo","louco","luxo","macaco","machado",
		                      "maior","manga","maracuja","marinheiro","matematica","matricula",
		                     "medico","menor","mensagem","mundo","milena","navio","nordeste","normal",
		                     "objeto","oficial","pacote","palhaco","papel","paulo","peixe",
		                     "queijo", "quinta","radio", "rato","sinal","sistema","tabela",
		                      "teclado", "uva","unidade", "vacina","vela", "vidro","watt",
		                      "xeque","xerox","you", "zebra", "zona", "vitoria"};

		//criando os arrays de strings que formarao as palavras compostas
		Palavra[] palavraArcoIris    = { new PalavraSimples("arco ") , new PalavraSimples("iris")};
		Palavra[] palavraAzulMarinho = { new PalavraSimples("azul ")  , new PalavraSimples("marinho")};
		Palavra[] palavraBeijaFlor   = { new PalavraSimples("beija")  , new PalavraSimples("flor")};
		Palavra[] palavraGuardaChuva = { new PalavraSimples("guarda") , new PalavraSimples("chuva")};
		Palavra[] palavraGuardaRoupa = { new PalavraSimples("guarda") , new PalavraSimples("roupa")};
		Palavra[] palavraGuardaSol   = { new PalavraSimples("guarda") , new PalavraSimples("sol")};

		//criando as palavras compostas
		PalavraComposta arcoIris    = new PalavraComposta(palavraArcoIris);
		PalavraComposta azulMarinho = new PalavraComposta(palavraAzulMarinho);
		PalavraComposta beijaFlor   = new PalavraComposta(palavraBeijaFlor);
		PalavraComposta guardaChuva = new PalavraComposta(palavraGuardaChuva);
		PalavraComposta guardaRoupa = new PalavraComposta(palavraGuardaRoupa);
		PalavraComposta guardaSol    = new PalavraComposta(palavraGuardaSol);

		//inserindo as palvras simples na lista
		for (int i=0; i< palavrasSimples.length ; i++){
		   lista.inserir(new PalavraSimples(palavrasSimples[i]));
		}
		//inserindo as palavras compostas na lista
		lista.inserir(arcoIris);
		lista.inserir(azulMarinho);
		lista.inserir(beijaFlor);
		lista.inserir(guardaChuva);
		lista.inserir(guardaRoupa);
		lista.inserir(guardaSol);
		celular = new PrevisaoTexto(new CadastroPalavras(lista));
  }        
	/**
	 * Metodo que retorna os caracteres associados a um digito
	 *
	 * @param  int      inteiro que representa a tecla escolhida
	 * @return char     conjunto de caracteres correspondentes a tecla escolhida
	 */
	public char[] caracteresDigito(int digito){
		char[] caracteres = null;
		switch (digito) {
			case 1:
				char[] c1 = {'a','b','c'};
				caracteres = c1;
				break;
			case 2:
			  char[] c2 = {'d','e','f'};
			  caracteres = c2;
			  break;
			case 3:
			  char[] c3 = {'g','h','i'};
			  caracteres = c3;
			  break;
			case 4:
			  char[] c4 = {'j','k','l'};
			  caracteres = c4;
			  break;
			case 5:
			  char[] c5 = {'m','n','o'};
			  caracteres = c5;
			  break;
			case 6:
			  char[] c6 = {'p','q','r'};
			  caracteres = c6;
			  break;
			case 7:
			  char[] c7 = {'s','t','u'};
			  caracteres = c7;
			  break;
			case 8:
			  char[] c8 = {'v','w','x'};
			  caracteres = c8;
			  break;
			case 9:
			  char[] c9 = {'y','z'};
			  caracteres = c9;
			  break;
	  }
	  return caracteres;
	}
	public static void main(String args[]) {
		InterfacePrevisaoTexto i = new InterfacePrevisaoTexto();
		String mensagem = "";
		Console io = new Console();
		io.println("*************************************************************"
					+ "*********");
		io.println("\t\tCin - Centro de informatica UFPE");
		io.println("*************************************************************"
					+ "*********");
		io.println("Esse sistema tem por objetivo simular um Sistema de mensagem de um Celular.");
		io.println("Cada tecla do teclado numerico corresponde a tres letras do alfabeto. ");
		io.println("Na primeira escolha de uma tecla, "
				   + "serao selecionadas todas as palavras ");
		io.println("que possuam a primeira letra igual a uma das letras "
				   + "correspondentes a essa tecla, ");
		 io.println("soh serah exibida a primeira letra da primeira palavra da lista. "
				   +"Na segunda escolha");
		io.println("serao selecionadas todas as palavras que "
				   +"alem de possuirem a primeira letra igual ");
		io.println("a uma das letras correspondente a primeira tecla digitada como tambm a segunda ");
		io.println("letra igual a uma das letras correspondentes a segunda tecla digitada, soh serao " );
		io.println("exibidas as duas primeiras letras da primeria palavra"
				   +"da lista e assim por diante. ");
		io.println("O processo ira se repetir ateh que uma palavra seja encontrada ");
		io.println("-------------------------------------------------------------"
					+ "---------");
		io.println(" 1 \t 2 \t 3 \t 4 \t 5 \t 6 \t 7 \t 8 \t 9 ");
		io.println("abc\tdef\tghi\tjkl\tmno\tpqr\tstu\tvwx\tyz");
		io.println("-------------------------------------------------------------"
					+ "---------");
		int count =0;
		boolean isErro = false;
		Palavra parcial = null;
		while (true) {
			io.println("\n\nDigite um numero para pesquisar a palavra:  ");
			String numDig = io.readString();
			//usando vector			
			//IListaPalavras p = new PalavrasList();

			//usando array
			IListaPalavras p = new PalavrasArray();	
				
			isErro = false;
			try{
			  p = i.preverTexto(numDig);

			} catch (PalavraNaoExisteException e){
				isErro = true;
				io.println("NAO EXISTE PALAVRA COM ESSA SEQUENCIA DE CARACTERES!!!");
				 io.println("\t" + mensagem);
				 i.indiceDigito =1;
				 count = 0;
			}
		  if(!isErro){
				io.println("\n\nResultado da Pesquisa:\n");

				//usando vector
				//((PalavrasList)p).getIterador().reset();

				//usando array
				((PalavrasArray)p).reset();
				
				//usando vector
				//parcial =  ((PalavrasList)p).getIterador().proxima();

				//usando array
				parcial =  ((PalavrasArray)p).proxima();
				String textoParcial="";


				if(p.getTamanho() !=1){

					if(parcial instanceof PalavraSimples){

					  textoParcial = i.preverParteTexto(((PalavraSimples)parcial).toString(),count+1);
					}  else if(parcial instanceof PalavraComposta){
					  textoParcial = i.preverParteTexto(((PalavraComposta)parcial).toString(),count+1);
					}

					io.println("\t" + textoParcial);
					count = count +1;
				}else if(p.getTamanho() ==1){
					 mensagem = mensagem +" "+ parcial.toString();
					 io.println("\t" + mensagem);
					 i.indiceDigito =1;
					 count = 0;

				} else {
				   io.println("\n\t!!Nenhuma palavra foi encontrada!!");

				}

			}//fecha o while
		  }//fecha o if
	  }//fecha o main  
   /**
	* Metodo que retorna a parte do texto da primeira palavra
	*
	* @param tecla          recebe o codigo numerico
	* @return char      do tipo vocabulario
	*/
   public String preverParteTexto(String s, int numIteracoes){
	  String resultado = "";
	  resultado = s.substring(0,numIteracoes);
	  return resultado;
   }      
  /**
	* Metodo que retorna um vocabulario filtrado
	* de acordo com um codigo numerico
	*
	* @param tecla          recebe o codigo numerico
	* @return pesquisa      do tipo vocabulario
	*/
   private IListaPalavras preverTexto(String numeroDigitado)throws PalavraNaoExisteException{

	  int digito = -1;

	  if(numeroDigitado != null){

		digito = Integer.parseInt(numeroDigitado.substring(0,1));
		if (indiceDigito == 1) {
		  this.filtro = celular.filtrar(caracteresDigito(digito),indiceDigito);

		  indiceDigito = indiceDigito+1;
		} else {

		  this.filtro = this.filtro.filtrar(caracteresDigito(digito),indiceDigito);

		  indiceDigito = indiceDigito + 1;
		}
	  } else {
		throw new ParametroNuloException("O numero digitado estah nulo");
	  }

	  return filtro;
	}
}
