package celular.palavras;

/**
 * Universidade Federal de Pernambuco
 * Centro de Informtica
 * Disciplina: Introducao a Programao
 * Projeto Celular
 *
 * Tipo: IListaPalavras
 *
 * @author: Milena Rocha Leite          (mrl@cin.ufpe.br)
 *
 * @since 2002
 * @version 1.0
 */
import celular.palavras.excecoes.PalavraNaoExisteException;
import celular.util.excecoes.ParametroNuloException;

public interface IListaPalavras {

	/**
	 * Mtodo que exclui uma palavra da colecao
	 *
	 * @param Palavra  palvra a ser excluda
	 */
	public void excluir(Palavra palavra) throws PalavraNaoExisteException;
	/**
	 * Mtodo que indica se uma palavra existe na colecao
	 *
	 * @param  Palavra   palavra a ser procurada
	 * @return boolean   boleano que informa se a palavra existe ou nao na colecao
	 */
	public boolean existe(Palavra palavra);
   /**
	* Mtodo que retorna todas as palavras do dicionario que possuem uma das
	* letras passadas da posicao indicada
	*
	* @param  char[]         characteres a serem verificados
	* @return ListaPalavras  lista de palavras que possuem um dos caracteres
	*                          passados
	*/
	public IListaPalavras filtrar(char[] letras,int posicaoAtual) throws PalavraNaoExisteException;
	/**
	 * Mtodo retorna o tamanho atual da colecao
	 *
	 * @return  int  tamanho da colecao
	 */
	public int getTamanho();
  /**
   * Mtodo que insere uma palavra na colecao
   *
   * @param Palvra   plavra a ser inserida
   */
	public void inserir(Palavra palavra);
   /**
	 * Mtodo que retorna a posicao de uma palavra na colecao
	 *
	 * @param Palavra    palavra a ser procurada
	 * @return int       que indica a posicao da palavra procurada
	 */
	public int posicao(Palavra chave);
}
