package celular.palavras;

/**
 * Universidade Federal de Pernambuco
 * Centro de Informtica
 * Disciplina: Introducao a Programao
 * Projeto Celular
 *
 * Tipo: PalavraComposta
 *
 * @author: Milena Rocha Leite          (mrl@cin.ufpe.br)
 *
 * @since 2002
 * @version 1.1
 */
import celular.util.excecoes.ParametroNuloException;
import celular.util.excecoes.ParametroInvalidoException;
import celular.util.Console;

public class PalavraComposta extends Palavra{

  private Palavra[] palavra;

  /**
   * Construtor da classe PalavraComposta
   *
   * @param  Palavra[]  conjunto de palavras que formam a palvra composta
   */
  public PalavraComposta(Palavra[] palavra) {
	if (palavra != null){
	   this.palavra = palavra;
	} else {
	  throw new ParametroNuloException("A String passada como parametro estah nula");
	}
  }  
  /**
   * Mtodo para a verificao de igualdade entre duas palavras
   *
   * @param  Palavra  palavra a ser comparada
   * @return boolean  boleano que indica se a palavra que chama o mtodo  igual
   *                  a palavra passada como parmetro
   */
  public boolean equals(Object palavra) {
	boolean retorno = false;
	Palavra temp = null;
	PalavraComposta pal = null;


	if( palavra instanceof PalavraComposta  ){
		if(((PalavraComposta)palavra).palavra.length == this.palavra.length){
			pal = (PalavraComposta)palavra;
			for(int i=0; i < this.palavra.length;i++ ){
			  temp = (Palavra)this.palavra[i];
			  if (temp.equals(pal.palavra[i]) ){
					retorno = true;
			  }else{
				  retorno = false;
				  i = this.palavra.length;
			  }
			}
		}
	}

	return retorno;
 } 
  /**
   * Mtodo que retorna a letra que est na posio passada como parmetro
   *
   * @param  int    posicao da letra procurada
   * @return char   retorna letra encontrada
   */
  public char getLetra(int posicao){
	char letra = '\n';
	PalavraSimples temp = null;
	int indice = 0;
	//usado para contar os tracinhos da palavra
	int tamanhoPalavrasAnteriores = 0;
	boolean achou = false;

	if(posicao <= this.getTamanho(this.palavra) && posicao != 0){
		while(indice < this.palavra.length && !achou){
			temp = (PalavraSimples)this.palavra[indice];

			//condicao usada para calcular a nova posicao de acordo com o numero
			//de palavras que jah foram lidas
			if(indice > 0){
			  for(int i=0; i< indice;i++){
				tamanhoPalavrasAnteriores = ((PalavraSimples)this.palavra[indice-1]).getPalavra().trim().length()+1;
			  }
			  posicao = posicao - tamanhoPalavrasAnteriores;
			}
			if(posicao  <= temp.getPalavra().trim().length() ){
			  if(posicao ==0 && indice >0){
				letra = '-';
			  }else{
				letra = temp.getPalavra().charAt(posicao-1);
			  }
			  achou = true;
			} else {
			  indice = indice + 1;
			}
		}
	}else {
	  letra = '\n';
	}
	return letra;
 } 
  /**
   * Mtodo que retorna o tamanho de todas as palvras que fazem parte do arrys
   * concatenas incluindo os tracinhos
   *
   * @return String  palavras concatenadas
   */
  private int getTamanho(Palavra[] pal) {
	int resultado = 0;
	int countTracinhos=0;
	if(this.palavra != null){
		for(int i =0;i < pal.length; i++){
		  resultado = resultado + ((PalavraSimples)pal[i]).getPalavra().trim().length();
		  countTracinhos = i;
		}

		resultado = resultado + countTracinhos;
	} else {
		throw new ParametroNuloException("O array de palavras estah nulo");
	}

	return resultado;
  }  
  public static void main(String args[]){

	Palavra[] palavraArcoIris    = { new PalavraSimples("arco ") , new PalavraSimples("iris"), new PalavraSimples("azul")};
	PalavraComposta arcoIris    = new PalavraComposta(palavraArcoIris);
	Palavra[] palavraArcoIrisTeste    = { new PalavraSimples("arco ") , new PalavraSimples("iri "), new PalavraSimples("azul")};
	PalavraComposta arcoIrisTeste    = new PalavraComposta(palavraArcoIrisTeste);
	System.out.println("As palavras sao iguai? " +arcoIris.equals(arcoIrisTeste));
	System.out.print(arcoIris.getLetra(1));
	System.out.print(arcoIris.getLetra(2));
	System.out.print(arcoIris.getLetra(3));
	System.out.print(arcoIris.getLetra(4));
	System.out.print(arcoIris.getLetra(5));
	System.out.print(arcoIris.getLetra(6));
	System.out.print(arcoIris.getLetra(7));
	System.out.print(arcoIris.getLetra(8));
	System.out.print(arcoIris.getLetra(9));
	System.out.print(arcoIris.getLetra(10));
	System.out.print(arcoIris.getLetra(11));
	System.out.print(arcoIris.getLetra(12));
	System.out.print(arcoIris.getLetra(13));
	System.out.print(arcoIris.getLetra(14));
	System.out.print(arcoIris.getLetra(15));
	char[] c = new char[3];
	c[0]='a';
	c[1]='b';
	c[2]='c';
	System.out.println("tem letra " + arcoIris.temLetras(c,2));
   System.out.println(arcoIris.getTamanho(palavraArcoIris));
	System.out.print("ToString " + arcoIris.toString());
  }  
  /**
   * Mtodo que verifica se a palavra possui em determinada posicao um dos
   * caracteres passados
   *
   * @boolean retorno        retorna a verificacao
   */
  public boolean temLetras(char[] letras, int posicao){
 	boolean retorno = false;
		char l = this.getLetra(posicao);
 	if (l!='\n') {
 	  for (int i = 0; i < letras.length && !retorno ; i++) {
			retorno = (l == letras[i]);
		  }
 	}
 	return retorno;
  }  
 /**
  * Mtodo que imprime a palavra
  *
  * @return String quer representa a palavra a ser impressa
  */
  public String toString(){
	String palavra="";

	for(int i=1; i <= this.getTamanho(this.palavra); i++ ){
	  palavra = palavra + this.getLetra(i);
	}
	return palavra;
  }  
}
