package celular.palavras.dados;

/**
 * Universidade Federal de Pernambuco
 * Centro de Informtica
 * Disciplina: Introducao a Programao
 * Projeto Celular
 *
 * Tipo: PalavrasArray
 *
 * Esta classe foi alterada para que fosse introduzido o conceito 
 * de concorrencia. O metodo posicao, instancia duas threads
 * do tipo Posicao. Cada thread vai procurar o indice em uma metade do
 * array.
 *
 * Foram alterados os seguintes mtodos:
 * 
 *	private synchronized void duplicarTamanho() {
 *	public synchronized void excluir(Palavra palavra) throws PalavraNaoExisteException{
 *  public synchronized boolean existe(Palavra chave) {
 *	public synchronized IListaPalavras filtrar(char[] letras,int posicaoAtual) throws PalavraNaoExisteException{
 *	public synchronized void inserir(Palavra palavra) {
 *	public synchronized int posicao(Palavra chave) {
 *	public synchronized Palavra proxima(){	
 *
 *
 * 
 * @author: Alberto Csar C. Frana                (accf2@cin.ufpe.br)
 * @author: Luciana Pereira Oliveira               (lpo@cin.ufpe.br)
 * @author: Milena Rocha Leite                     (mrl@cin.ufpe.br)
 * @author: Ives Jos de Albuquerque Macdo Jnior (ijamj@cin.ufpe.br)
 *
 *
 * @since 2002
 * @version 1.3
 */
import celular.palavras.IListaPalavras;
import celular.palavras.Palavra;
import celular.palavras.PalavraSimples;
import celular.palavras.excecoes.PalavraNaoExisteException;
import celular.util.excecoes.ParametroNuloException;


public class PalavrasArray implements IListaPalavras {

	private Palavra[] elementos;
	private int proxima;  //indica o proximo indice que pode receber um objeto
	private int maximo;  //indica o tamanho maximo do array de objetos

	private final static byte NAO_EXISTE = -1;
	private final static int TAMANHO_MAXIMO_INICIAL = 5;

	private int indiceParaIteracao;

	/**
	 * Construtor da classe PalavrasArray
	 *
	 */
	public PalavrasArray() {
		this.reset();
		maximo = TAMANHO_MAXIMO_INICIAL;
		elementos = new Palavra[maximo];
		proxima = 0;
	}
	/**
	 * Mtodo que duplica o tamanho do array.
	 */
	private synchronized void duplicarTamanho() {
		Palavra[] temp = new Palavra[this.maximo * 2];

		for (int i = 0; i < this.maximo; i = i + 1) {
		   temp[i] = this.elementos[i];
		}
		this.elementos = temp;
		this.maximo = this.maximo * 2;
	}
	/**
	 * Mtodo que exclui uma palavra do array de palavras
	 *
	 * @param Palavra  palvra a ser excluda
	 */
	public synchronized void excluir(Palavra palavra) throws PalavraNaoExisteException{

		if (palavra != null){
			int i = this.posicao(palavra);
			if (i == NAO_EXISTE) {
				throw new PalavraNaoExisteException("A palavra a ser excluda no existe");
			} else {
				elementos[i] = null;
				elementos[i] = elementos[proxima - 1];
				elementos[proxima - 1] = null;
				proxima = proxima - 1;
			}
		} else {
			throw new ParametroNuloException("A palavra a ser excluda est nula");
		}
	}
	/**
	 * Mtodo que indica se uma palavra existe na colecao
	 *
	 * @param  Palavra   palavra a ser procurada
	 * @return boolean   boleano que informa se a palavra existe ou nao na colecao
	 */
	 public synchronized boolean existe(Palavra chave) {
		 boolean resposta = false;
		 if (chave != null){
			 if (this.posicao(chave) == NAO_EXISTE) {
				resposta = false;
			 } else {
				resposta = true;
			 }
		 } else {
			 throw new ParametroNuloException("A palavra passada como parametro estah"
											  + "nula");
		 }
		 return resposta;
	}
	/**
	 * Mtodo que retorna todas as palavras do dicionario que possuem uma das
	 * letras passadas da posicao indicada
	 *
	 * @param  char[]         characteres a serem verificados
	 * @return ListaPalavras  lista de palavras que possuem um dos caracteres
	 *                          passados
	 */
	public synchronized IListaPalavras filtrar(char[] letras,int posicaoAtual) throws PalavraNaoExisteException{
		PalavrasArray temp = new PalavrasArray();
		Palavra palavra;
		for (int i = 0 ; i < proxima; i++){
		  palavra = elementos[i];
		  if(palavra.temLetras(letras,posicaoAtual)){
			temp.inserir(palavra);
		  }
		}
		if(temp.getTamanho() == 0){
			throw new PalavraNaoExisteException("Nao existe uma palavra com essa sequencia"
											  +"de caracteres");
		}
		return temp;
	}
	/**
	 * Mtodo retorna o tamanho atual do array
	 *
	 * @return  int  tamanho atual do array
	 */
	public int getTamanho() {
		return proxima;
	}
	/**
	 * Mtodo que insere uma palavra na colecao
	 *
	 * @param Palvra   plavra a ser inserida
	 */
	public synchronized void inserir(Palavra palavra) {
		if (palavra != null){
			if (this.proxima == this.maximo) {
				this.duplicarTamanho();
			}
			this.elementos[proxima] = palavra;
			this.proxima = this.proxima + 1;
		} else {
			throw new ParametroNuloException("A palavra a ser inserida est nula");
		}
	}
  //main para verificacao de alguns metodos
  public static void main(String args[]) {

	PalavraSimples p1,p2,p3,p4,p6,p5;
	try{
		p1 = new PalavraSimples("casa");
		PalavrasArray lista = new PalavrasArray();
		lista.inserir(p1);
		p2 = new PalavraSimples("carro");
		lista.inserir(p2);
		p3 = new PalavraSimples("estrada");
		lista.inserir(p3);
		p4 = new PalavraSimples("onda");
		lista.inserir(p4);
		p5 = new PalavraSimples("roda");
		lista.inserir(p5);
		p6 = new PalavraSimples("praia");
		lista.inserir(p6);
		System.out.println("tam" +lista.getTamanho());
	//    System.out.println("get 0"+ (lista.proxima()).getPalavra());
		lista.excluir(p4);
		System.out.println("tam: "+lista.getTamanho());
		lista.reset();
	//    System.out.println("get 0!!!"+ (lista.proxima()).getPalavra());
		System.out.println("Verifica se existe casa "+lista.existe(new PalavraSimples("casa")));
		System.out.println("Verifica se existe mm "+lista.existe(new PalavraSimples("mm")));
		System.out.println("Verifica posica de casa "+lista.posicao(new PalavraSimples("casa")));
	  } catch(ParametroNuloException e){
		System.out.println("Algum parametro estah nulo");
	  } catch(PalavraNaoExisteException e){
		System.out.println("Algum palavra nao existe");
	  }
  }  
	/**
	 * Mtodo que retorna a posicao de uma palavra no conjunto de maneira
	 * concorrente buscando nas duas metades do array "ao mesmo tempo"
	 *
	 * @param Palavra    palavra a ser procurada
	 * @return int       que indica a posicao da palavra procurada
	 */
	public synchronized int posicao(Palavra chave) {
/*
		int i = 0;
		boolean achou = false;
		if (chave != null){
			while(!achou && i < proxima) {
				if (chave.equals(this.elementos[i])) {
					achou = true;
				} else {
					i = i + 1;
				}
			}
			if (!achou) {
			   i = NAO_EXISTE;
			}
		} else {
		  i = NAO_EXISTE;
		}

		return i;
*/
		int ret = PalavrasArray.NAO_EXISTE;

		if (chave != null) {
			int meio = this.proxima/2;
			Posicao thread1 = new Posicao(this.elementos, chave, 0, meio);
			Posicao thread2 = new Posicao(this.elementos, chave, meio, proxima);

			thread1.setFriend(thread2);
			thread2.setFriend(thread1);

			thread1.start();
			thread2.start();

			try {
				thread1.join();
				thread2.join();
			} catch (InterruptedException e) {}

			if (thread1.getIndice() != meio) {
				ret = thread1.getIndice();
			} else {
				ret = thread2.getIndice();
			}

		} else ret = PalavrasArray.NAO_EXISTE;

		return ret = 0;
	}
	/**
	 * Mtodo que retorna a proxima palavra da lista de acordo com o proxima de
	 * navegao dentro da lista
	 *
	 * @return Palavra
	 */
	public synchronized Palavra proxima(){
		Palavra retorno = null;
		if (indiceParaIteracao < proxima) {
			retorno = elementos[indiceParaIteracao];
			indiceParaIteracao = indiceParaIteracao + 1;
		}
		return retorno;
	}
	/**
	 *  Mtodo que reseta o proxima de navegacao dentro da lista
	 */
	public void reset(){
		this.indiceParaIteracao = 0;
	}
}
