package celular.palavras.dados;

/**
 * Universidade Federal de Pernambuco
 * Centro de Informtica
 * Disciplina: Introducao a Programao
 * Projeto Celular
 *
 * Tipo: PalavrasArray
 *
 * @author: Alberto Csar C. Frana                (accf2@cin.ufpe.br)
 * @author: Luciana Pereira Oliveira               (lpo@cin.ufpe.br)
 * @author: Milena Rocha Leite                     (mrl@cin.ufpe.br)
 * @author: Ives Jos de Albuquerque Macdo Jnior (ijamj@cin.ufpe.br)
 *
 * @since 2002
 * @version 1.3
 */
import celular.palavras.IListaPalavras;
import celular.palavras.Palavra;
import celular.palavras.PalavraSimples;
import celular.palavras.excecoes.PalavraNaoExisteException;
import celular.util.excecoes.ParametroNuloException;


/**
 * Classe que possiblita a busca pela posicao de uma palavra no conjunto PalavrasArray
 * de maneira concorrente buscando nas duas metades do array "ao mesmo tempo"
 */
class Posicao extends Thread {


	/**
	 * Atributo que identifica o indice encontrado na busca pela palavra chave
	 */
	private int indice;

	/**
	 * Atributo que identifica o outro Thread, que busca na outra metade do array
	 */
	private Posicao friend;

	/**
	 * Atributo que identifica se o outro Thread jah encontrou a palavra chave
	 */
	private boolean stop;


	/**
	 * Atributo que identifica o array no qual serah buscada a palavra chave
	 */
	private Palavra[] palavras;

	/**
	 * Atributo que identifica a palavra chave a ser buscada
	 */
	private Palavra chave;

	/**
	 * Atributo que identifica o indice inicial da busca
	 */
	private int inicio;

	/**
	 * Atributo que identifica o indice final da busca
	 */
	private int fim;


	/**
	 * Construtor da classe Posicao que inicializa o
	 * objeto com os atributos passados como parametros
	 *
	 * @param palavras  Conjunto de dados com as palavras a serem analisadas
	 * @param chave     Palavra-chave a ser buscada
	 * @param inicio    Indice pelo qual a busca se iniciarah
	 * @param fim       Indice maximo no qual a busca terminarah
	 */
	public Posicao(Palavra[] palavras, Palavra chave, int inicio, int fim) {
		this.palavras = palavras;
		this.chave = chave;
		this.inicio = inicio;
		this.fim = fim;
		this.indice = this.inicio;
		this.friend = this;
		this.stop = false;
	}
	/**
	 * Metodo que possibilita recuperar o indice encontrado na busca
	 *
	 * @return O indice encontrado na busca
	 */
	public int getIndice() {
		return this.indice;
	}
	/**
	 * Metodo que efetiva a busca concorrente em uma das metades do array,
	 * no intervalo inteiro especificado por [inicio, fim)
	 */
	public void run() {

		 if (this.chave != null) {
			 boolean achou = false;

			 while ((!this.stop) && (!achou) && (this.indice < this.fim)) {

				if (this.chave.equals(palavras[this.indice])) {
					achou = true;
					this.friend.stop = true;
				} else this.indice = this.indice + 1;

				if (this.stop) {
					this.indice = this.fim;
				}

			 }

		 } else this.indice = this.fim;

	}
	/**
	 * Metodo que possibilita setar o Thread
	 * que buscarah na outra metade do Conjunto de Dados
	 *
	 * @param friend O Thread que buscarah na outra metade do Conjunto de Dados
	 */
	public void setFriend(Posicao friend) {

		if (friend != null) {
			this.friend = friend;
		} else {
			this.friend = this;
		}

	}
}
